package cc.toph.simplycompat.item;

import cc.toph.simplycompat.registry.ConfigRegistry.WEAPON_ATTRIBUTES;
import com.google.common.base.Suppliers;
import net.minecraft.core.registries.BuiltInRegistries;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.Items;
import net.minecraft.world.item.Tier;
import net.minecraft.world.item.crafting.Ingredient;
import org.jetbrains.annotations.NotNull;

import java.util.function.Supplier;

public enum SimplyCompatToolMaterials implements Tier {
  COPPER(1, 125, 4.5F, 1.0F, WEAPON_ATTRIBUTES.COPPER, 8, Items.COPPER_INGOT),
  STEEL(2, 600, 6.5F, 2.5F, WEAPON_ATTRIBUTES.STEEL, 12, Items.DIAMOND);

  private final int level;
  private final int uses;
  private final float speed;
  private final float attackDamageBonus;
  private final float damageModifier;
  private final int enchantmentValue;
  private final Supplier<Ingredient> repairIngredient;

  SimplyCompatToolMaterials(
          int level,
          int uses,
          float speed,
          float attackDamageBonus,
          float damageModifier,
          int enchantmentValue,
          Item... repairIngredient
  ) {
    this.level = level;
    this.uses = uses;
    this.speed = speed;
    this.attackDamageBonus = attackDamageBonus;
    this.enchantmentValue = enchantmentValue;
    this.repairIngredient = Suppliers.memoize(() -> Ingredient.of(repairIngredient));
    this.damageModifier = damageModifier;
  }


  /**
   * Returns the name of the enum constant in lowercase.
   *
   * @return the name of the enum constant as a lowercase string
   */
  public String getName() {
    return this.name().toLowerCase();
  }

  /**
   * Returns RepairIngredient ResourceLocation as a string
   *
   * @return String ResourceLocation Path (mod:item)
   */
  public String getIdentifier() {
    return BuiltInRegistries.ITEM.getKey(
            this.repairIngredient.get().getItems()[0].getItem()
    ).toString();
  }

  @Override
  public int getUses() {
    return uses;
  }

  @Override
  public float getSpeed() {
    return speed;
  }

  @Override
  public float getAttackDamageBonus() {
    return attackDamageBonus;
  }

  /**
   * Retrieves the base damage modifier associated with the tool material.
   * <p>
   * Simply Swords specific value, not related to {@link Tier#getAttackDamageBonus()}
   *
   * @return the base damage modifier as a float value
   */
  public float getDamageModifier() {
    return damageModifier;
  }

  @Override
  public int getLevel() {
    return level;
  }

  @Override
  public int getEnchantmentValue() {
    return enchantmentValue;
  }

  @Override
  public @NotNull Ingredient getRepairIngredient() {
    return this.repairIngredient.get();
  }
}
