package cc.toph.simplycompat.registry;

import cc.toph.simplycompat.SimplyCompat;
import cc.toph.simplycompat.item.SimplyCompatToolMaterials;
import cc.toph.simplycompat.util.WeaponType;
import dev.architectury.registry.registries.DeferredRegister;
import net.minecraft.core.registries.Registries;
import net.minecraft.world.item.Item;
import net.sweenus.simplyswords.item.SimplySwordsSwordItem;

public class ItemsRegistry {
  public static final DeferredRegister<Item> ITEMS = DeferredRegister.create(
          SimplyCompat.MOD_ID,
          Registries.ITEM
  );

  private static void registerSword(
          SimplyCompatToolMaterials material,
          WeaponType type
  ) {

    float finalDamage = material.getDamageModifier() + type.getPositiveModifier() - type.getNegativeModifier();
    String name = material.getName() + "_" + type.getWeaponNameSuffix();

    ITEMS.register(name, () ->
            new SimplySwordsSwordItem(material, (int) finalDamage, type.getAttackSpeed(), material.getIdentifier())
    );
  }

  /**
   * Registers all sword items. Iterate through all available weapon types
   * and tool materials. For each combination, the corresponding sword
   * is registered using the private {@code registerSword} method.
   * <br>
   * This ensures that each defined material and weapon type combination is
   * registered within the item registry for use within the mod.
   */
  public static void registerAllSwords() {
    // TODO: Register Compat Swords

    // Register "Vanilla" swords
    for (WeaponType type : WeaponType.values()) {
      for (SimplyCompatToolMaterials material : SimplyCompatToolMaterials.values()) {
        registerSword(material, type);
      }
    }

    ITEMS.register();
  }
}
